@@ -1,17 +1,17 @@
-#!/usr/bin/python
+#! /usr/bin/python3
 # -*- coding: utf-8 -*-
 from argparse import RawTextHelpFormatter
 import readline, glob
 import sys, time, os
 import subprocess
-import xml.dom.minidom
+import xml.etree.ElementTree as ET
 import re
 import argparse
-import argcomplete
 import threading
 import itertools
 import tempfile
 import shutil
+import json
 from multiprocessing import Process
 
 services = {}
@@ -57,12 +57,14 @@ banner = colors.red + r"""
         ╚═════╝ ╚═╝  ╚═╝ ╚═════╝    ╚═╝   ╚══════╝╚══════╝╚═╝     ╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   
                                                                                    
 """+'\n' \
-+ '\n brutespray.py v1.6.0' \
++ '\n brutespray.py v1.6.8' \
 + '\n Created by: Shane Young/@x90skysn3k && Jacob Robles/@shellfail' \
 + '\n Inspired by: Leon Johnson/@sho-luv' \
 + '\n Credit to Medusa: JoMo-Kun / Foofus Networks <jmk@foofus.net>\n' + colors.normal
 #ascii art by: Cara Pearson
 
+quiet_banner = colors.red + '~ BruteSpray ~' + colors.normal
+
 class tabCompleter(object):
 
     def pathCompleter(self,text,state):
@@ -74,47 +76,47 @@ def interactive():
     t = tabCompleter()
     singluser = ""
     if args.interactive is True:
-        print colors.white + "\n\nWelcome to interactive mode!\n\n" + colors.normal
-        print colors.red + "WARNING:" + colors.white + " Leaving an option blank will leave it empty and refer to default\n\n" + colors.normal
-        print "Available services to brute-force:"
+        print(colors.white + "\n\nWelcome to interactive mode!\n\n" + colors.normal)
+        print(colors.red + "WARNING:" + colors.white + " Leaving an option blank will leave it empty and refer to default\n\n" + colors.normal)
+        print("Available services to brute-force:")
         for serv in services:
             srv = serv
             for prt in services[serv]:
                 iplist = services[serv][prt]
                 port = prt
                 plist = len(iplist)
-                print "Service: " + colors.green + str(serv) + colors.normal + " on port " + colors.red + str(port) + colors.normal + " with " + colors.red + str(plist) + colors.normal + " hosts"
+                print("Service: " + colors.green + str(serv) + colors.normal + " on port " + colors.red + str(port) + colors.normal + " with " + colors.red + str(plist) + colors.normal + " hosts")
 
-        args.service = raw_input('\n' + colors.lightblue + 'Enter services you want to brute - default all (ssh,ftp,etc): ' + colors.red)
-        
-        args.threads = raw_input(colors.lightblue + 'Enter the number of parallel threads (default is 2): ' + colors.red)
+        args.service = input('\n' + colors.lightblue + 'Enter services you want to brute - default all (ssh,ftp,etc): ' + colors.red)
 
-        args.hosts = raw_input(colors.lightblue + 'Enter the number of parallel hosts to scan per service (default is 1): ' + colors.red)
+        args.threads = input(colors.lightblue + 'Enter the number of parallel threads (default is 2): ' + colors.red)
+
+        args.hosts = input(colors.lightblue + 'Enter the number of parallel hosts to scan per service (default is 1): ' + colors.red)
 
         if args.passlist is None or args.userlist is None:
-            customword = raw_input(colors.lightblue + 'Would you like to specify a wordlist? (y/n): ' + colors.red)
+            customword = input(colors.lightblue + 'Would you like to specify a wordlist? (y/n): ' + colors.red)
         if customword == "y":
             readline.set_completer_delims('\t')
             readline.parse_and_bind("tab: complete")
             readline.set_completer(t.pathCompleter)
             if args.userlist is None and args.username is None:
-                args.userlist = raw_input(colors.lightblue + 'Enter a userlist you would like to use: ' + colors.red)
+                args.userlist = input(colors.lightblue + 'Enter a userlist you would like to use: ' + colors.red)
                 if args.userlist == "":
                     args.userlist = None
             if args.passlist is None and args.password is None:
-                args.passlist = raw_input(colors.lightblue + 'Enter a passlist you would like to use: ' + colors.red)
+                args.passlist = input(colors.lightblue + 'Enter a passlist you would like to use: ' + colors.red)
                 if args.passlist == "":
                     args.passlist = None
-            
+
         if args.username is None or args.password is None: 
-            singluser = raw_input(colors.lightblue + 'Would to specify a single username or password (y/n): ' + colors.red)
+            singluser = input(colors.lightblue + 'Would to specify a single username or password (y/n): ' + colors.red)
         if singluser == "y":
             if args.username is None and args.userlist is None:
-                args.username = raw_input(colors.lightblue + 'Enter a username: ' + colors.red)
+                args.username = input(colors.lightblue + 'Enter a username: ' + colors.red)
                 if args.username == "":
                     args.username = None
             if args.password is None and args.passlist is None:
-                args.password = raw_input(colors.lightblue + 'Enter a password: ' + colors.red)
+                args.password = input(colors.lightblue + 'Enter a password: ' + colors.red)
                 if args.password == "":
                     args.password = None
 
@@ -125,137 +127,125 @@ def interactive():
         if args.hosts == "":
             args.hosts = "1"
 
-    print colors.normal
+    print(colors.normal)
+
+NAME_MAP = {"ms-sql-s": "mssql",
+            "microsoft-ds": "smbnt",
+            "pcanywheredata": "pcanywhere",
+            "postgresql": "postgres",
+            "shell": "rsh",
+            "exec": "rexec",
+            "login": "rlogin",
+            "smtps": "smtp",
+            "submission": "smtp",
+            "imaps": "imap",
+            "pop3s": "pop3",
+            "iss-realsecure": "vmauthd",
+            "snmptrap": "snmp"}
 
 def make_dic_gnmap():
     global loading
     global services
+
+    supported = ['ssh','ftp','postgres','telnet','mysql','ms-sql-s','shell',
+                 'vnc','imap','imaps','nntp','pcanywheredata','pop3','pop3s',
+                 'exec','login','microsoft-ds','smtp', 'smtps','submission',
+                 'svn','iss-realsecure','snmptrap','snmp']
+
+
     port = None
     with open(args.file, 'r') as nmap_file:
         for line in nmap_file:
-            supported = ['ssh','ftp','postgres','telnet','mysql','ms-sql-s','shell','vnc','imap','imaps','nntp','pcanywheredata','pop3','pop3s','exec','login','microsoft-ds','smtp', 'smtps','submission','svn','iss-realsecure','snmptrap','snmp']
             for name in supported:
                 matches = re.compile(r'([0-9][0-9]*)/open/[a-z][a-z]*//' + name)
                 try:
                     port =  matches.findall(line)[0]
                 except:
                     continue
-    
+
                 ip = re.findall( r'[0-9]+(?:\.[0-9]+){3}', line)
                 tmp_ports = matches.findall(line)
+
                 for tmp_port in tmp_ports:
-                        if name =="ms-sql-s":
-                            name = "mssql"
-                        if name == "microsoft-ds":
-                            name = "smbnt"
-                        if name == "pcanywheredata":
-                            name = "pcanywhere"
-                        if name == "shell":
-                            name = "rsh"
-                        if name == "exec":
-                            name = "rexec"
-                        if name == "login":
-                            name = "rlogin"
-                        if name == "smtps" or name == "submission":
-                            name = "smtp"
-                        if name == "imaps":
-                            name = "imap"
-                        if name == "pop3s":
-                            name = "pop3"
-                        if name == "iss-realsecure":
-                            name = "vmauthd"
-                        if name == "snmptrap":
-                            name = "snmp"
-                        if name in services:
-                            if tmp_port in services[name]:
-                                services[name][tmp_port] += ip
-                            else:
-                                services[name][tmp_port] = ip
+
+                    name = NAME_MAP.get(name, name)
+
+                    if name in services:
+                        if tmp_port in services[name]:
+                            services[name][tmp_port] += ip
                         else:
-                            services[name] = {tmp_port:ip}
+                            services[name][tmp_port] = ip
+                    else:
+                        services[name] = {tmp_port:ip}
 
     loading = True
 
 def make_dic_xml():
     global loading
     global services
-    supported = ['ssh','ftp','postgresql','telnet','mysql','ms-sql-s','rsh','vnc','imap','imaps','nntp','pcanywheredata','pop3','pop3s','exec','login','microsoft-ds','smtp','smtps','submission','svn','iss-realsecure','snmptrap','snmp']
-    doc = xml.dom.minidom.parse(args.file)
-    for host in doc.getElementsByTagName("host"):
-        try:
-            address = host.getElementsByTagName("address")[0]
-            ip = address.getAttribute("addr")
-            eip = ip.encode("utf8")
-            iplist = eip.split(',')
-        except:
-            # move to the next host
-            continue
-        try:
-            status = host.getElementsByTagName("status")[0]
-            state = status.getAttribute("state")
-        except:
-            state = ""
-        try:
-            ports = host.getElementsByTagName("ports")[0]
-            ports = ports.getElementsByTagName("port")
-        except:
-            continue
-
-        for port in ports:
-            pn = port.getAttribute("portid")
-            state_el = port.getElementsByTagName("state")[0]
-            state = state_el.getAttribute("state")
-            if state == "open":
+    supported = ['ssh','ftp','postgresql','telnet','mysql','ms-sql-s','rsh',
+                 'vnc','imap','imaps','nntp','pcanywheredata','pop3','pop3s',
+                 'exec','login','microsoft-ds','smtp','smtps','submission',
+                 'svn','iss-realsecure','snmptrap','snmp']
+    tree = ET.parse(args.file)
+    root = tree.getroot()
+    for host in root.iter('host'):
+        ipaddr = host.find('address').attrib['addr']
+        for port in host.iter('port'):
+            cstate = port.find('state').attrib['state']
+            if cstate == "open":
                 try:
-                    service = port.getElementsByTagName("service")[0]
-                    port_name = service.getAttribute("name")
+                    name = port.find('service').attrib['name']
+                    tmp_port = port.attrib['portid']
+                    iplist = ipaddr.split(',')
                 except:
-                    service = ""
-                    port_name = ""
-                    product_descr = ""
-                    product_ver = ""
-                    product_extra = ""
-                name = port_name.encode("utf-8")
-                tmp_port = pn.encode("utf-8")
+                    continue
                 if name in supported:
-                    if name == "postgresql":
-                        name = "postgres"
-                    if name =="ms-sql-s":
-                        name = "mssql"
-                    if name == "microsoft-ds":
-                        name = "smbnt"
-                    if name == "pcanywheredata":
-                        name = "pcanywhere"
-                    if name == "shell":
-                        name = "rsh"
-                    if name == "exec":
-                        name = "rexec"
-                    if name == "login":
-                        name = "rlogin"
-                    if name == "smtps" or name == "submission":
-                        name = "smtp"
-                    if name == "imaps":
-                        name = "imap"
-                    if name == "pop3s":
-                        name = "pop3"
-                    if name == "iss-realsecure":
-                        name = "vmauthd"
-                    if name == "snmptrap":
-                        name = "snmp"
+                    name = NAME_MAP.get(name, name)
                     if name in services:
                         if tmp_port in services[name]:
                             services[name][tmp_port] += iplist
-                        else:   
-                         services[name][tmp_port] = iplist
+                        else:
+                            services[name][tmp_port] = iplist
                     else:
                         services[name] = {tmp_port:iplist}
-    loading = True        
 
+    loading = True
+
+def make_dic_json():
+    global loading
+    global services
+
+    supported = ['ssh','ftp','postgres','telnet','mysql','ms-sql-s','shell',
+                 'vnc','imap','imaps','nntp','pcanywheredata','pop3','pop3s',
+                 'exec','login','microsoft-ds','smtp', 'smtps','submission',
+                 'svn','iss-realsecure','snmptrap','snmp']
+
+    with open(args.file, "r") as jsonlines_file:
+        for line in jsonlines_file:
+            data = json.loads(line)
+            try:
+                host, port, name = data["host"], data["port"], data["service"]
+                
+                if name in supported:
+                    name = NAME_MAP.get(name, name) 
+                    if name not in services:
+                        services[name] = {}
+                    if port not in services[name]:
+                        services[name][port] = []
+                    if host not in services[name][port]:
+                        services[name][port].append(host)
+            except KeyError as e:
+                sys.stderr.write("\n[!] Field: " + str(e) + "is missing")
+                sys.stderr.write("\n[!] Please provide the json fields. ")               
+                continue
+    loading = True 
 
 def brute(service,port,fname,output):
-
     if args.userlist is None and args.username is None:
-        userlist = 'wordlist/'+service+'/user'
+        userlist = '/usr/share/brutespray/wordlist/'+service+'/user'
+        if not os.path.exists(userlist):
+            userlist = 'wordlist/'+service+'/user'
         uarg = '-U'
     elif args.userlist:
         userlist = args.userlist
@@ -265,7 +255,9 @@ def brute(service,port,fname,output):
         uarg = '-u'
 
     if args.passlist is None and args.password is None:
-        passlist = 'wordlist/'+service+'/password'
+        passlist = '/usr/share/brutespray/wordlist/'+service+'/password'
+        if not os.path.exists(passlist):
+            passlist = 'wordlist/'+service+'/password'
         parg = '-P'
     elif args.passlist:
         passlist = args.passlist
@@ -273,37 +265,44 @@ def brute(service,port,fname,output):
     elif args.password:
         passlist = args.password
         parg = '-p'
-    
+
     if args.continuous:
         cont = ''
     else:
         cont = '-F'
-    
-    p = subprocess.Popen(['medusa', '-H', fname, uarg, userlist, parg, passlist, '-M', service, '-t', args.threads, '-n', port, '-T', args.hosts, cont], stdout=subprocess.PIPE, stderr=subprocess.PIPE, bufsize=-1)
+    if service == "smtp":
+        aarg = "-m"
+        auth = "AUTH:LOGIN"
+    else:
+        aarg = ''
+        auth = ''
+
+    p = subprocess.Popen(['medusa', '-b', '-H', fname, uarg, userlist, parg, passlist, '-M', service, '-t', args.threads, '-n', port, '-T', args.hosts, cont, aarg, auth], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=-1)
 
     out = "[" + colors.green + "+" + colors.normal + "] "
-    output_file = output + '/' + service + '-success.txt'
+    output_file = output + '/' + port + '-' + service + '-success.txt'
     
  
     for line in iter(p.stdout.readline, b''):
-        print line,
+        print(line.decode('utf-8').strip('\n'))
         sys.stdout.flush()
         time.sleep(0.0001)
-        if 'SUCCESS' in line:
+        if 'SUCCESS' in line.decode('utf-8'):
             f = open(output_file, 'a')
-            f.write(out + line)
-            f.close()    
-   
+            f.write(out + line.decode('utf-8'))
+            f.close()
+
 def animate():
-        t_end = time.time() + 2
-        for c in itertools.cycle(['|', '/', '-', '\\']):
-            if not time.time() < t_end:
-                break
-            sys.stdout.write('\rStarting to brute, please make sure to use the right amount of threads(-t) and parallel hosts(-T)...  ' + c)
-            sys.stdout.flush()
-            time.sleep(0.1)
-        sys.stdout.write('\n\nBrute-Forcing...     \n') 
-        time.sleep(1)
+    sys.stdout.write('\rStarting to brute, please make sure to use the right amount of ' + colors.green + 'threads(-t)' + colors.normal + ' and ' + colors.green + 'parallel hosts(-T)' + colors.normal + '...  \n')
+    t_end = time.time() + 2
+    for c in itertools.cycle(['|', '/', '-', '\\']):
+        if not time.time() < t_end:
+            break
+        sys.stdout.write('\rOutput will be written to the folder: ./' + colors.green + args.output + colors.normal + "/ "+ c)
+        sys.stdout.flush()
+        time.sleep(0.1)
+    sys.stdout.write('\n\nBrute-Forcing...     \n')
+    time.sleep(1)
 
 def loading():
     for c in itertools.cycle(['|', '/', '-', '\\']):
@@ -313,15 +312,38 @@ def loading():
         sys.stdout.flush()
         time.sleep(0.01)
 
+def getInput(filename):
+    in_format = None
+    with open(filename) as f:
+        line = f.readlines()
+        if filename.endswith("gnmap"):
+            in_format = "gnmap"
+        if filename.endswith("json"):
+            in_format = "json"
+        if filename.endswith("xml"):
+            in_format = "xml"
+        if '{' in line[0]:
+            in_format = "json"
+        if '# Nmap' in line[0] and not 'Nmap' in line[1]:
+            in_format = "gnmap"
+        if '<?xml ' in line[0]:
+            in_format = "xml"
+        if in_format is None:
+            print('File is not correct format!\n')
+            sys.exit(0)
+
+    return in_format 
+
 def parse_args():
-    
+
     parser = argparse.ArgumentParser(formatter_class=RawTextHelpFormatter, description=\
- 
+
     "Usage: python brutespray.py <OPTIONS> \n")
 
     menu_group = parser.add_argument_group(colors.lightblue + 'Menu Options' + colors.normal)
-    
-    menu_group.add_argument('-f', '--file', help="GNMAP or XML file to parse", required=True)
+
+    #menu_group.add_argument('-f', '--file', help="GNMAP or XML file to parse", required=False, default=None)
+    menu_group.add_argument('-f', '--file', help="GNMAP, JSON or XML file to parse", required=False, default=None)
     menu_group.add_argument('-o', '--output', help="Directory containing successful attempts", default="brutespray-output")
     menu_group.add_argument('-s', '--service', help="specify service to attack", default="all")
     menu_group.add_argument('-t', '--threads', help="number of medusa threads", default="2")
@@ -332,17 +354,33 @@ def parse_args():
     menu_group.add_argument('-p', '--password', help="specify a single password", default=None)
     menu_group.add_argument('-c', '--continuous', help="keep brute-forcing after success", default=False, action='store_true')
     menu_group.add_argument('-i', '--interactive', help="interactive mode", default=False, action='store_true')    
+    menu_group.add_argument('-m', '--modules', help="dump a list of available modules to brute", default=False, action='store_true')    
+    menu_group.add_argument('-q', '--quiet', help="supress banner", default=False, action='store_true')   
 
-    argcomplete.autocomplete(parser)    
     args = parser.parse_args()
-    
+
+    if args.file is None and args.modules is False:
+        parser.error("argument -f/--file is required")
     return args
 
 if __name__ == "__main__":
-    print(banner)
     args = parse_args()
+    
+    if args.quiet == False:
+        print(banner)
+    else:
+        print(quiet_banner)
 
+    supported = ['ssh','ftp','telnet','vnc','mssql','mysql','postgresql','rsh',
+                'imap','nntp','pcanywhere','pop3',
+                'rexec','rlogin','smbnt','smtp',
+                'svn','vmauthd','snmp']
     #temporary directory for ip addresses
+
+    if args.modules is True:
+        print(colors.lightblue + "Supported Services:\n" + colors.green)
+        print(('\n'.join(supported)))
+        print(colors.normal + "\n") 
     try:
         tmppath = tempfile.mkdtemp(prefix="brutespray-tmp")
     except:
@@ -355,23 +393,43 @@ if __name__ == "__main__":
     if os.system("command -v medusa > /dev/null") != 0:
         sys.stderr.write("Command medusa not found. Please install medusa before using brutespray")
         exit(3)
-        
-    try:
-        t = threading.Thread(target=loading)
-        t.start()
-        doc = xml.dom.minidom.parse(args.file)
-        make_dic_xml()
-    except:
-        make_dic_gnmap()
-    
-    if args.interactive is True:
-        interactive()
-    
-    animate()
-    
-    if services == {}:
-        print "\nNo brutable services found.\n Please check your Nmap file."
- 
+
+    if args.file is None:
+        sys.exit(0)
+
+    if args.passlist and not os.path.isfile(args.passlist):
+        sys.stderr.write("Passlist given does not exist. Please check your file or path\n")
+        exit(3)
+
+    if args.userlist and not os.path.isfile(args.userlist):
+        sys.stderr.write("Userlist given does not exist. Please check your file or path\n")
+        exit(3)
+
+    if os.path.isfile(args.file):        
+        try:
+            t = threading.Thread(target=loading)
+            t.start()
+            in_format = getInput(args.file)
+            {
+                "gnmap": make_dic_gnmap,
+                "xml":   make_dic_xml,
+                "json":  make_dic_json
+            }[in_format]()
+        except:
+            print("\nFormat failed!\n")
+            loading = True
+            sys.exit(0)
+
+        if args.interactive is True:
+            interactive()
+
+        animate()
+
+        if services == {}:
+            print("\nNo brutable services found.\n Please check your Nmap file.")
+    else:
+        print("\nError loading file, please check your filename.")
+
     to_scan = args.service.split(',')
     for service in services:
         if service in to_scan or to_scan == ['all']:
